package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.text.DateFormat;
import java.util.ArrayList;
import com.bpcl.facades.payment.refund.RefundMsdData;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import com.bpcl.lpg.core.utill.BillDeskMsgUtils;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.servicelayer.media.NoDataAvailableException;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.text.ParseException;
import com.bpcl.core.custom.exception.BpclInvalidPaymentResponseException;
import com.bpcl.core.custom.exception.BpclUnAuthorizedPaymentResponse;
import com.bpcl.lpg.core.enums.LpgPaymentStatusEnum;
import java.text.MessageFormat;
import com.bpcl.lpg.core.enums.RefillBookingStatusEnum;
import java.text.SimpleDateFormat;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.payment.billdesk.LPGRefundService;
import com.bpcl.lpg.core.pojo.BillDeskResponseMsgData;
import java.util.List;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentService;
import com.bpcl.lpg.core.payment.billdesk.LPGPaymentStatusCheckService;
import com.bpcl.lpg.core.payment.billdesk.dao.LPGPaymentInfoDao;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class DailyTransactionReconciliationJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private LPGPaymentInfoDao lpgPaymentInfoDao;
    private LPGPaymentStatusCheckService lpgPaymentStatusCheckService;
    private LPGPaymentService lpgPaymentService;
    private KeyGenerator paymentTransactionIdGenerator;
    private Converter<List<String>, BillDeskResponseMsgData> queryPaymentResponseDataConverter;
    private LPGRefundService lpgRefundService;
    private NotificationEngineService notificationEngineService;
    private ConfigurationService configurationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronJobModel) {
        try {
            final List<LpgPaymentInfoModel> transactionList = this.prepareTransactionList(cronJobModel);
            DailyTransactionReconciliationJob.LOG.info("Number of transactions to be processed by: {}", (Object)transactionList.size());
            final SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
            transactionList.forEach(model -> {
                final String transactionID = model.getTransactionID();
                DailyTransactionReconciliationJob.LOG.info("Processing transaction {}", transactionID);
                try {
                    this.modelService.refresh((Object)model);
                    if (null != model.getRefillBookingStatus() && model.getRefillBookingStatus().equals((Object)RefillBookingStatusEnum.SUCCESSFUL)) {
                        DailyTransactionReconciliationJob.LOG.info(MessageFormat.format("Transaction {0} has Refill booking status as successful,No refund required", model.getTransactionID()));
                        model.setTransactionStatus(LpgPaymentStatusEnum.SUCCESSFUL);
                        this.modelService.save((Object)model);
                    }
                    else {
                        this.performReconciliation(transactionID, model, format);
                    }
                }
                catch (final BpclUnAuthorizedPaymentResponse | BpclInvalidPaymentResponseException | ParseException e) {
                    DailyTransactionReconciliationJob.LOG.error("Exception occurred: {} for transaction {}", e.getCause(), transactionID);
                }
                catch (final JsonProcessingException e2) {
                    DailyTransactionReconciliationJob.LOG.error("JsonProcessingException occurred: {} for transaction {}", e2.getCause(), transactionID);
                    throw new RuntimeException(e2);
                }
                return;
            });
        }
        catch (final NoDataAvailableException ndaEx) {
            DailyTransactionReconciliationJob.LOG.info(ndaEx.getMessage());
        }
        catch (final Exception ex) {
            DailyTransactionReconciliationJob.LOG.error(ex.getMessage(), ex.getCause());
            final String userId = this.configurationService.getConfiguration().getString("lpg.alert.email.receiver.id");
            final String body = MessageFormat.format("Daily Transaction reconciliation cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Daily Transaction Reconciliation Job Alert", body, userId);
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
        return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
    }
    
    private void performReconciliation(final String transactionID, final LpgPaymentInfoModel transaction, final SimpleDateFormat format) throws BpclUnAuthorizedPaymentResponse, BpclInvalidPaymentResponseException, ParseException, JsonProcessingException {
        final String msg = this.lpgPaymentStatusCheckService.getPaymentStatusFromBillDeskByTxnID(transactionID);
        if (StringUtils.isNotBlank((CharSequence)msg)) {
            final boolean authorizedResponse = this.lpgPaymentService.isAuthorizedResponse(msg);
            if (!authorizedResponse) {
                DailyTransactionReconciliationJob.LOG.error("UnAuthorized response from the provider");
                throw new BpclUnAuthorizedPaymentResponse("UnAuthorized response from the provider");
            }
            final List<String> messageList = BillDeskMsgUtils.convertMsgToAttributes(msg, 33);
            final BillDeskResponseMsgData responseMsgData = (BillDeskResponseMsgData)this.queryPaymentResponseDataConverter.convert((Object)messageList);
            if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"0300")) {
                DailyTransactionReconciliationJob.LOG.info("Payment is success for txn {}", transactionID);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
                if (this.isValidRefundStatus(responseMsgData.getRefundStatus())) {
                    DailyTransactionReconciliationJob.LOG.debug(MessageFormat.format("Refund already initiated for txn id {0} ", transactionID));
                    transaction.setTransactionStatus(LpgPaymentStatusEnum.REFUND_SUCCESSFUL);
                    transaction.setIsRefunded(Boolean.valueOf(true));
                }
                else {
                    DailyTransactionReconciliationJob.LOG.info("Initiating refund for txn {}", transactionID);
                    final RefundMsdData refundMsdData = this.getRefundData(responseMsgData);
                    this.lpgRefundService.processRefund(refundMsdData, transaction);
                    this.modelService.refresh((Object)transaction);
                    if (StringUtils.isNotEmpty((CharSequence)transaction.getRefundProcessStatus()) && !StringUtils.equals((CharSequence)"NA", (CharSequence)transaction.getRefundProcessStatus()) && StringUtils.equals((CharSequence)"Y", (CharSequence)transaction.getRefundProcessStatus()) && StringUtils.isNotEmpty((CharSequence)transaction.getBilldeskRefundID()) && !StringUtils.equals((CharSequence)"NA", (CharSequence)transaction.getBilldeskRefundID())) {
                        final String content = MessageFormat.format(this.configurationService.getConfiguration().getString("lpg.customer.refund.notification"), String.valueOf(transaction.getRefundAmount()), transaction.getBankReferenceNo(), format.format(new Date()));
                        final String templateId = this.configurationService.getConfiguration().getString("lpg.ym.v2.cashmemo.sms.templateId");
                        final String mobileNumber = transaction.getAccount().getBpclCustomer().getUid();
                        final Map<String, String> params = new HashMap<String, String>();
                        params.put("1", String.valueOf(transaction.getRefundAmount()));
                        params.put("2", transaction.getBankReferenceNo());
                        params.put("3", format.format(new Date()));
                        this.lpgV2NotificationEngineService.sendSMS(templateId, mobileNumber, (Map)params);
                    }
                }
            }
            else if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"0001")) {
                DailyTransactionReconciliationJob.LOG.info("changing status to Failed for txn {}", transactionID);
                transaction.setTransactionStatus(LpgPaymentStatusEnum.FAILED);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
            }
            else if (StringUtils.equalsIgnoreCase((CharSequence)responseMsgData.getAuthStatus(), (CharSequence)"NA")) {
                DailyTransactionReconciliationJob.LOG.info("changing status to Error for txn {}", transactionID);
                transaction.setTransactionStatus(LpgPaymentStatusEnum.ERROR);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
            }
            else if (this.isMaximumDaysToRetryExceeded(transaction)) {
                DailyTransactionReconciliationJob.LOG.info("As maximum retry for transaction exceeded changing status to aborted for txn {}", transactionID);
                transaction.setTransactionStatus(LpgPaymentStatusEnum.ABORTED);
                this.lpgPaymentService.updatePaymentDetailsForRecon(responseMsgData, transaction);
            }
            this.modelService.save((Object)transaction);
        }
    }
    
    private RefundMsdData getRefundData(final BillDeskResponseMsgData responseMsgData) {
        final RefundMsdData refundMsdData = new RefundMsdData();
        refundMsdData.setBilldeskTxnID(responseMsgData.getTxnReferenceNo());
        refundMsdData.setRefundRefNo((String)this.paymentTransactionIdGenerator.generate());
        refundMsdData.setTxnAmount(Double.valueOf(responseMsgData.getTxnAmount()).toString());
        refundMsdData.setTxnDate(responseMsgData.getTxnDate());
        refundMsdData.setTransactionID(responseMsgData.getCustomerID());
        return refundMsdData;
    }
    
    private boolean isValidRefundStatus(final String refundStatus) {
        return StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0699") || StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0799") || StringUtils.equalsIgnoreCase((CharSequence)refundStatus, (CharSequence)"0799");
    }
    
    private List<LpgPaymentInfoModel> prepareTransactionList(final BpclCronjobModel cronJobModel) throws ParseException {
        final Date currentDate = new Date();
        DailyTransactionReconciliationJob.LOG.info("Cronjob run time:: {}", currentDate);
        Date filrerStartDate = cronJobModel.getFilterStartDate();
        DailyTransactionReconciliationJob.LOG.info("Cronjob Filter Start Date time:: {}", filrerStartDate);
        Date filrerEndDate = cronJobModel.getFilterEndDate();
        DailyTransactionReconciliationJob.LOG.info("Cronjob Filter End Date time:: {}", filrerEndDate);
        if (filrerEndDate == null) {
            final DateFormat dateFormat = BPCLDateUtils.getSimpleDateFormat("dd/MM/yyyy");
            filrerEndDate = dateFormat.parse(dateFormat.format(currentDate));
        }
        if (filrerStartDate == null) {
            filrerStartDate = new Date(filrerEndDate.getTime() - 432000000L);
        }
        DailyTransactionReconciliationJob.LOG.info("Start date::: {}", filrerStartDate);
        DailyTransactionReconciliationJob.LOG.info("End date::: {}", filrerEndDate);
        final List<String> enums = new ArrayList<String>();
        enums.add(LpgPaymentStatusEnum.IN_PROGRESS.getCode());
        enums.add(LpgPaymentStatusEnum.INITIATED.getCode());
        return this.lpgPaymentInfoDao.getTransactionsByStatusForDateRange(filrerStartDate, filrerEndDate, (List)enums);
    }
    
    private boolean isMaximumDaysToRetryExceeded(final LpgPaymentInfoModel transaction) throws ParseException {
        final Date currentDate = new Date();
        final DateFormat dateFormat = BPCLDateUtils.getSimpleDateFormat("dd/MM/yyyy");
        final Date todayDate = dateFormat.parse(dateFormat.format(currentDate));
        final Date transactionDate = dateFormat.parse(dateFormat.format(transaction.getCreationtime()));
        final Date maxAllowedDate = new Date(todayDate.getTime() - 432000000L);
        return transactionDate.equals(maxAllowedDate) || transactionDate.before(maxAllowedDate);
    }
    
    public boolean isAbortable() {
        return true;
    }
    
    public void setLpgPaymentInfoDao(final LPGPaymentInfoDao lpgPaymentInfoDao) {
        this.lpgPaymentInfoDao = lpgPaymentInfoDao;
    }
    
    public void setLpgPaymentStatusCheckService(final LPGPaymentStatusCheckService lpgPaymentStatusCheckService) {
        this.lpgPaymentStatusCheckService = lpgPaymentStatusCheckService;
    }
    
    public void setLpgPaymentService(final LPGPaymentService lpgPaymentService) {
        this.lpgPaymentService = lpgPaymentService;
    }
    
    public void setQueryPaymentResponseDataConverter(final Converter<List<String>, BillDeskResponseMsgData> queryPaymentResponseDataConverter) {
        this.queryPaymentResponseDataConverter = queryPaymentResponseDataConverter;
    }
    
    public KeyGenerator getPaymentTransactionIdGenerator() {
        return this.paymentTransactionIdGenerator;
    }
    
    public void setPaymentTransactionIdGenerator(final KeyGenerator paymentTransactionIdGenerator) {
        this.paymentTransactionIdGenerator = paymentTransactionIdGenerator;
    }
    
    public LPGRefundService getLpgRefundService() {
        return this.lpgRefundService;
    }
    
    public void setLpgRefundService(final LPGRefundService lpgRefundService) {
        this.lpgRefundService = lpgRefundService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(DailyTransactionReconciliationJob.class);
    }
}
